/*
 * file: XxApplication.cpp
 *
 * This file is part of the XxWinLib library which is developed to support
 * the development of NetStreamer. This file is distributed under the
 * GNU GENERAL PUBLIC LICENSE, see the accompanying COPYING file.
 *
 * Copyright (C) 1997 Rolf Fokkens
 *
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <fcntl.h>

#include "XxBlocker.h"
#include "XxTimer.h"
#include "XxApplication.h"
#include "XxWindow.h"
#include "XxAutoInit.h"

#include "XxButton.h"

class XxAppBlocker : public XxBlocker {
protected:
public:
    XxAppBlocker (int XxFd);
    ~XxAppBlocker (void);

    virtual void     GetRWFlags (int &rFlag, int &wFlag);
    virtual XxAction GetAction  (int &rFlag, int &wFlag);

    virtual void HandleAction (XxAction Action);

    long GetInCount  (void) { return 0; };
    long GetOutCount (void) { return 0; };
};

NULLINIT (XxApplication *pDummyApplication);
NULLINIT (XxAppBlocker  *pDummyAppBlocker);

class InitClass {
private:
    int Dummy;
public:
    InitClass (void) {
        XxApplication::Init ();
    };
    ~InitClass (void) {
        delete pDummyApplication;
    };
};

AUTOINIT InitClass InitObject;

void XxApplication::Init (void)
{
    if (pDummyApplication == NULL) {
        XxBlocker::Init ();
        pDummyApplication = new XxApplication;

        AUTOINIT_TRACE ("XxApplication::Init");
    };
};

XxAppBlocker::XxAppBlocker (int XxFd)
{
    SetFd (XxFd);
    SetStatus (StatOpen);
};

XxAppBlocker::~XxAppBlocker (void)
{
    XCloseDisplay (XxApplication::GetDisplay ());
    SetFd (-1);
};

void XxAppBlocker::GetRWFlags (int &rFlag, int &wFlag)
{
    rFlag = 1;
    wFlag = 0;
};

XxBlocker::XxAction XxAppBlocker::GetAction  (int &rFlag, int &wFlag)
{
    return (rFlag ? ActRead : ActNone);
};

void XxAppBlocker::HandleAction (XxAction Action)
{
    XxApplication::HandleEvents ();
};

int      XxApplication::InitDone = 0;
Display *XxApplication::XxDisplay;
int      XxApplication::XxScreen, XxApplication::XxWidth;
int      XxApplication::XxHeight, XxApplication::XxDepth;
Atom     XxApplication::WmDeleteWindow;
long     XxApplication::BGW [8];
Window   XxApplication::ButtWindow = 0;

/*
long XxApplication::GetBGW (int i)
{
    return BGW [i & 7];
};
*/

XxApplication::XxApplication (void)
{
    int    i, Fd;
    XColor color;
    Status s;

    if (InitDone) return;

    XxDisplay  = XOpenDisplay  (NULL);
    XxScreen   = DefaultScreen (XxDisplay);

    XxWidth    = DisplayWidth  (XxDisplay, XxScreen);
    XxHeight   = DisplayHeight (XxDisplay, XxScreen);
    XxDepth    = DefaultDepth  (XxDisplay, XxScreen);

    WmDeleteWindow = XInternAtom   (XxDisplay, "WM_DELETE_WINDOW", False);

    InitDone = 1;

    BGW [0] = BlackPixel (GetDisplay (), GetScreen ());
    BGW [7] = WhitePixel (GetDisplay (), GetScreen ());

    for (i = 1; i < 7; i++) {
        color.red   = (i * 0x2000);
        color.green = (i * 0x2000);
        color.blue  = (i * 0x2000);

        XAllocColor (XxDisplay, DefaultColormap (XxDisplay, XxScreen), &color);

        BGW [i] = color.pixel;
    };

    Fd               = ConnectionNumber (XxDisplay);
    pDummyAppBlocker = new XxAppBlocker (Fd);

    fcntl (Fd, F_SETFL, O_NONBLOCK);
};

XxApplication::~XxApplication (void)
{
    delete pDummyAppBlocker;
};

int XxApplication::HandleEvents (void)
{
    XEvent   event;
    XxWindow *pWindow;
    Window    PassOnWindow;

    for (;;) {
        XNextEvent (GetDisplay (), &event);

        switch (event.type) {
        case ButtonPress:
            PassOnWindow = event.xexpose.window;
            ButtWindow   = PassOnWindow;
            break;
        case ButtonRelease:
            PassOnWindow = ButtWindow;
            ButtWindow   = 0;
            break;
        case ClientMessage:
        case Expose:
        case EnterNotify:
        case LeaveNotify:
        case ResizeRequest:
            PassOnWindow = event.xexpose.window;
            break;
        default:
            PassOnWindow = 0;
            break;
        };

        if (PassOnWindow != 0) {
            pWindow = WindowsOwner.GetFirst ();
            while (pWindow != NULL) {
                if (pWindow->Xid == PassOnWindow) {
                    pWindow->HandleEvent (event);
                    break;
                };
                pWindow = pWindow->WindowsMember.GetNext ();
            };
        };
        if (!XPending (GetDisplay ())) break;
    };
};

void XxApplication::Flush (void)
{
    XxCore::DoPreFlushActions ();

    XFlush (GetDisplay ());
};

int XxApplication::MainLoop (void)
{
    XEvent              event;
    XxWindow           *pWindow;
    XxBlocker          *pBlocker;
    int                 mSec;
    XxBlocker::XxAction bAction;
    int                 mSize, lSize = - 1;

    XxTimer::RefreshMsCurTime ();

    for (;;) {
        Flush ();

        if (XxCore::GetCount () == 0) break;

        XxCore::CleanupWasteBasket ();

        mSec = XxBlocker::HandleTimers ();

        Flush ();

        XxBlocker::HandleBlocker (mSec);
    };

    return 0;
};

